@@ -3,12 +3,12 @@
  * pgtclCmds.c
  *	  C functions which implement pg_* tcl commands
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/interfaces/libpgtcl/Attic/pgtclCmds.c,v 1.69 2002/10/17 14:51:50 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpgtcl/pgtclCmds.c,v 1.74.2.1 2003/11/08 20:34:42 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -23,7 +23,7 @@
 /*
  * Local function forward declarations
  */
-static int execute_put_values(Tcl_Interp *interp, char *array_varname,
+static int execute_put_values(Tcl_Interp *interp, CONST84 char *array_varname,
 				   PGresult *result, int tupno);
 
 
@@ -241,7 +241,7 @@ tcl_value(char *value)
  **********************************/
 
 int
-Pg_conndefaults(ClientData cData, Tcl_Interp *interp, int argc, char **argv)
+Pg_conndefaults(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PQconninfoOption *options = PQconndefaults();
 	PQconninfoOption *option;
@@ -287,13 +287,13 @@ Pg_conndefaults(ClientData cData, Tcl_Interp *interp, int argc, char **argv)
  **********************************/
 
 int
-Pg_connect(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_connect(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
-	char	   *pghost = NULL;
-	char	   *pgtty = NULL;
-	char	   *pgport = NULL;
-	char	   *pgoptions = NULL;
-	char	   *dbName;
+	const char *pghost = NULL;
+	const char *pgtty = NULL;
+	const char *pgport = NULL;
+	const char *pgoptions = NULL;
+	const char *dbName;
 	int			i;
 	PGconn	   *conn;
 
@@ -397,7 +397,7 @@ Pg_connect(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
  **********************************/
 
 int
-Pg_disconnect(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_disconnect(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	Tcl_Channel conn_chan;
@@ -436,7 +436,7 @@ Pg_disconnect(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
  **********************************/
 
 int
-Pg_exec(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_exec(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	Pg_ConnectionId *connid;
 	PGconn	   *conn;
@@ -506,6 +506,8 @@ Pg_exec(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 
 	-numTuples	the number of tuples in the query
 
+	-cmdTuples	the number of tuples affected by the query
+
 	-numAttrs	returns the number of attributes returned by the query
 
 	-assign arrayName
@@ -536,13 +538,13 @@ Pg_exec(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 
  **********************************/
 int
-Pg_result(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_result(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGresult   *result;
-	char	   *opt;
+	const char *opt;
 	int			i;
 	int			tupno;
-	char	   *arrVar;
+	CONST84 char *arrVar;
 	char		nameBuffer[256];
 	const char *appendstr;
 
@@ -591,6 +593,11 @@ Pg_result(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 		sprintf(interp->result, "%d", PQntuples(result));
 		return TCL_OK;
 	}
+	else if (strcmp(opt, "-cmdTuples") == 0)
+	{
+		sprintf(interp->result, "%s", PQcmdTuples(result));
+		return TCL_OK;
+	}
 	else if (strcmp(opt, "-numAttrs") == 0)
 	{
 		sprintf(interp->result, "%d", PQnfields(result));
@@ -768,6 +775,7 @@ Pg_result_errReturn:
 					 "\t-conn\n",
 					 "\t-oid\n",
 					 "\t-numTuples\n",
+					 "\t-cmdTuples\n",
 					 "\t-numAttrs\n"
 					 "\t-assign arrayVarName\n",
 					 "\t-assignbyidx arrayVarName ?appendstr?\n",
@@ -796,7 +804,7 @@ Pg_result_errReturn:
  **********************************/
 
 int
-Pg_execute(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_execute(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	Pg_ConnectionId *connid;
 	PGconn	   *conn;
@@ -805,8 +813,8 @@ Pg_execute(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 	int			tupno;
 	int			ntup;
 	int			loop_rc;
-	char	   *oid_varname = NULL;
-	char	   *array_varname = NULL;
+	CONST84 char *oid_varname = NULL;
+	CONST84 char *array_varname = NULL;
 	char		buf[64];
 
 	char	   *usage = "Wrong # of arguments\n"
@@ -1017,7 +1025,7 @@ Pg_execute(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
  column names, or into an array indexed by the column names.
  **********************************/
 static int
-execute_put_values(Tcl_Interp *interp, char *array_varname,
+execute_put_values(Tcl_Interp *interp, CONST84 char *array_varname,
 				   PGresult *result, int tupno)
 {
 	int			i;
@@ -1063,7 +1071,7 @@ execute_put_values(Tcl_Interp *interp, char *array_varname,
 **********************/
 
 int
-Pg_lo_open(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_open(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			lobjId;
@@ -1133,7 +1141,7 @@ Pg_lo_open(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 
 **********************/
 int
-Pg_lo_close(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_close(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			fd;
@@ -1207,22 +1215,30 @@ Pg_lo_read(ClientData cData, Tcl_Interp *interp, int objc,
 	buf = ckalloc(len + 1);
 
 	nbytes = lo_read(conn, fd, buf, len);
-	bufObj = Tcl_NewStringObj(buf, nbytes);
 
-	if (Tcl_ObjSetVar2(interp, bufVar, NULL, bufObj,
-					   TCL_LEAVE_ERR_MSG | TCL_PARSE_PART1) == NULL)
-		rc = TCL_ERROR;
-	else
+	if (nbytes >= 0)
+	{
+#if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 1 || TCL_MAJOR_VERSION > 8
+		bufObj = Tcl_NewByteArrayObj(buf, nbytes);
+#else
+		bufObj = Tcl_NewStringObj(buf, nbytes);
+#endif
+
+		if (Tcl_ObjSetVar2(interp, bufVar, NULL, bufObj,
+						   TCL_LEAVE_ERR_MSG | TCL_PARSE_PART1) == NULL)
+			rc = TCL_ERROR;
+	}
+
+	if (rc == TCL_OK)
 		Tcl_SetObjResult(interp, Tcl_NewIntObj(nbytes));
 
 	ckfree(buf);
 	return rc;
-
 }
 
 #else
 int
-Pg_lo_read(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_read(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			fd;
@@ -1257,7 +1273,9 @@ Pg_lo_read(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 
 	nbytes = lo_read(conn, fd, buf, len);
 
-	Tcl_SetVar(interp, bufVar, buf, TCL_LEAVE_ERR_MSG);
+	if (nbytes >= 0)
+		Tcl_SetVar(interp, bufVar, buf, TCL_LEAVE_ERR_MSG);
+
 	sprintf(interp->result, "%d", nbytes);
 	ckfree(buf);
 	return TCL_OK;
@@ -1299,7 +1317,11 @@ Pg_lo_write(ClientData cData, Tcl_Interp *interp, int objc,
 	if (Tcl_GetIntFromObj(interp, objv[2], &fd) != TCL_OK)
 		return TCL_ERROR;
 
+#if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 1 || TCL_MAJOR_VERSION > 8
+	buf = Tcl_GetByteArrayFromObj(objv[3], &nbytes);
+#else
 	buf = Tcl_GetStringFromObj(objv[3], &nbytes);
+#endif
 
 	if (Tcl_GetIntFromObj(interp, objv[4], &len) != TCL_OK)
 		return TCL_ERROR;
@@ -1320,7 +1342,7 @@ Pg_lo_write(ClientData cData, Tcl_Interp *interp, int objc,
 
 #else
 int
-Pg_lo_write(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_write(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	char	   *buf;
@@ -1368,11 +1390,11 @@ whence can be either
 "SEEK_CUR", "SEEK_END", or "SEEK_SET"
 ***********************************/
 int
-Pg_lo_lseek(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_lseek(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			fd;
-	char	   *whenceStr;
+	const char *whenceStr;
 	int			offset,
 				whence;
 
@@ -1421,7 +1443,7 @@ for now, we don't support any additional storage managers.
 
 ***********************************/
 int
-Pg_lo_creat(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_creat(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	char	   *modeStr;
@@ -1439,7 +1461,7 @@ Pg_lo_creat(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 	if (conn == (PGconn *) NULL)
 		return TCL_ERROR;
 
-	modeStr = argv[2];
+	modeStr = strdup(argv[2]);
 
 	modeWord = strtok(modeStr, "|");
 	if (strcmp(modeWord, "INV_READ") == 0)
@@ -1451,6 +1473,7 @@ Pg_lo_creat(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 		Tcl_AppendResult(interp,
 						 "invalid mode argument to Pg_lo_creat\nmode argument must be some OR'd combination of INV_READ, and INV_WRITE",
 						 0);
+		free(modeStr);
 		return TCL_ERROR;
 	}
 
@@ -1465,10 +1488,12 @@ Pg_lo_creat(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 			Tcl_AppendResult(interp,
 							 "invalid mode argument to Pg_lo_creat\nmode argument must be some OR'd combination of INV_READ, INV_WRITE",
 							 0);
+			free(modeStr);
 			return TCL_ERROR;
 		}
 	}
 	sprintf(interp->result, "%d", lo_creat(conn, mode));
+	free(modeStr);
 	return TCL_OK;
 }
 
@@ -1481,7 +1506,7 @@ Pg_lo_tell
 
 ***********************************/
 int
-Pg_lo_tell(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_tell(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			fd;
@@ -1514,7 +1539,7 @@ Pg_lo_unlink
 
 ***********************************/
 int
-Pg_lo_unlink(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_unlink(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
 	int			lobjId;
@@ -1556,10 +1581,10 @@ Pg_lo_import
 ***********************************/
 
 int
-Pg_lo_import(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_import(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
-	char	   *filename;
+	const char *filename;
 	Oid			lobjId;
 
 	if (argc != 3)
@@ -1599,10 +1624,10 @@ Pg_lo_export
 ***********************************/
 
 int
-Pg_lo_export(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_lo_export(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	PGconn	   *conn;
-	char	   *filename;
+	const char *filename;
 	Oid			lobjId;
 	int			retval;
 
@@ -1651,7 +1676,7 @@ Pg_lo_export(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
  **********************************/
 
 int
-Pg_select(ClientData cData, Tcl_Interp *interp, int argc, char **argv)
+Pg_select(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	Pg_ConnectionId *connid;
 	PGconn	   *conn;
@@ -1808,9 +1833,9 @@ Pg_listen
    vwait or update can be used to enter the Tcl event loop.
 ***********************************/
 int
-Pg_listen(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_listen(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
-	char	   *origrelname;
+	const char *origrelname;
 	char	   *caserelname;
 	char	   *callback = NULL;
 	Pg_TclNotifies *notifies;
@@ -1852,7 +1877,7 @@ Pg_listen(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
 	else
 	{
 		/* Downcase it */
-		char	   *rels = origrelname;
+		const char *rels = origrelname;
 		char	   *reld = caserelname;
 
 		while (*rels)
@@ -1989,7 +2014,7 @@ Pg_on_connection_loss
    vwait or update can be used to enter the Tcl event loop.
 ***********************************/
 int
-Pg_on_connection_loss(ClientData cData, Tcl_Interp *interp, int argc, char *argv[])
+Pg_on_connection_loss(ClientData cData, Tcl_Interp *interp, int argc, CONST84 char *argv[])
 {
 	char	   *callback = NULL;
 	Pg_TclNotifies *notifies;
